﻿using System;
using System.Collections.Generic;
using System.Linq;
using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.Common.Entities;
using Vintagestory.API.Datastructures;
using Vintagestory.API.MathTools;
using Vintagestory.GameContent;

namespace fartempleaxe.src
{
    class ClassMechanicalAxe : ModSystem
    {
        public override void Start(ICoreAPI api)
        {
            base.Start(api);
            api.RegisterItemClass("axemechanical", typeof(axemechanical));
        }
    }

	public class axemechanical : Item
	{

		// Token: 0x0600179E RID: 6046 RVA: 0x000D8910 File Offset: 0x000D6B10
		static axemechanical()
		{
			axemechanical.dustParticles.ParticleModel = EnumParticleModel.Quad;
			axemechanical.dustParticles.AddPos.Set(1.0, 1.0, 1.0);
			axemechanical.dustParticles.MinQuantity = 2f;
			axemechanical.dustParticles.AddQuantity = 12f;
			axemechanical.dustParticles.LifeLength = 4f;
			axemechanical.dustParticles.MinSize = 0.2f;
			axemechanical.dustParticles.MaxSize = 0.5f;
			axemechanical.dustParticles.MinVelocity.Set(-0.4f, -0.4f, -0.4f);
			axemechanical.dustParticles.AddVelocity.Set(0.8f, 1.2f, 0.8f);
			axemechanical.dustParticles.DieOnRainHeightmap = false;
			axemechanical.dustParticles.WindAffectednes = 0.5f;
		}

		//

		public override string GetHeldTpUseAnimation(ItemSlot activeHotbarSlot, Entity byEntity)
		{
			return "spearhit";
		}

		public override bool OnHeldInteractCancel(float secondsUsed, ItemSlot slot, EntityAgent byEntity, BlockSelection blockSel, EntitySelection entitySel, EnumItemUseCancelReason cancelReason)
		{
			return false;

		}



		public override void OnHeldInteractStart(ItemSlot itemslot, EntityAgent byEntity, BlockSelection blockSel, EntitySelection entitySel, bool firstEvent, ref EnumHandHandling handling)
		{


			byEntity.Attributes.SetInt("didattack", 0);

			itemslot.Itemstack.Collectible.AttackRange = AttackRange * 1.4f;




			byEntity.World.RegisterCallback((dt) =>
			{
				IPlayer byPlayer = (byEntity as EntityPlayer).Player;
				if (byPlayer == null) return;

				if (byEntity.Controls.HandUse == EnumHandInteract.HeldItemInteract)
				{
					var pitch = (byEntity as EntityPlayer).talkUtil.pitchModifier;
					byPlayer.Entity.World.PlaySoundAt(new AssetLocation("sounds/player/strike"), byPlayer.Entity, byPlayer, pitch * 0.9f + (float)api.World.Rand.NextDouble() * 0.2f, 12, 0.35f);
				}
			}, 464);

			handling = EnumHandHandling.PreventDefault;

		}

		public override bool OnHeldInteractStep(float secondsPassed, ItemSlot slot, EntityAgent byEntity, BlockSelection blockSel, EntitySelection entitySel)
		{
			float backwards = -Math.Min(0.8f, 3 * secondsPassed);
			float stab = Math.Min(1.2f, 20 * Math.Max(0, secondsPassed - 0.25f));

			if (byEntity.World.Side == EnumAppSide.Client)
			{
				IClientWorldAccessor world = byEntity.World as IClientWorldAccessor;
				ModelTransform tf = new ModelTransform();
				tf.EnsureDefaultValues();


				float sum = stab + backwards;
				float ztranslation = Math.Min(0.2f, 1.5f * secondsPassed);
				float easeout = Math.Max(0, 2 * (secondsPassed - 1));

				if (secondsPassed > 0.4f) sum = Math.Max(0, sum - easeout);
				ztranslation = Math.Max(0, ztranslation - easeout);

				tf.Rotation.Set(-sum * 190, sum * 0, -sum * 0);
				tf.Translation.Set(-2f * sum, ztranslation * 1f, -sum * 2f * 1f);


				byEntity.Controls.UsingHeldItemTransformAfter = tf;


				if (stab > 1.15f && byEntity.Attributes.GetInt("didattack") == 0)
				{
					world.TryAttackEntity(entitySel);
					byEntity.Attributes.SetInt("didattack", 1);
					world.AddCameraShake(0.25f);
				}
			}
			return secondsPassed < 1.4f;

		}


		public override void OnHeldInteractStop(float secondsUsed, ItemSlot itemslot, EntityAgent byEntity, BlockSelection blockSel, EntitySelection entitySel)
		{
			itemslot.Itemstack.Collectible.AttackRange = AttackRange / 1.4f;
		}
		//

		// Token: 0x0600179F RID: 6047 RVA: 0x0007E87D File Offset: 0x0007CA7D
		public override void OnHeldAttackStart(ItemSlot slot, EntityAgent byEntity, BlockSelection blockSel, EntitySelection entitySel, ref EnumHandHandling handling)
		{
			base.OnHeldAttackStart(slot, byEntity, blockSel, entitySel, ref handling);
		}

		// Token: 0x060017A0 RID: 6048 RVA: 0x000D8ABD File Offset: 0x000D6CBD
		public override bool OnHeldAttackStep(float secondsPassed, ItemSlot slot, EntityAgent byEntity, BlockSelection blockSelection, EntitySelection entitySel)
		{
			return base.OnHeldAttackStep(secondsPassed, slot, byEntity, blockSelection, entitySel);
		}

		// Token: 0x060017A1 RID: 6049 RVA: 0x000D8ACC File Offset: 0x000D6CCC
		public override float OnBlockBreaking(IPlayer player, BlockSelection blockSel, ItemSlot itemslot, float remainingResistance, float dt, int counter)
		{
			ITreeAttribute tempAttr = itemslot.Itemstack.TempAttributes;
			int posx = tempAttr.GetInt("lastposX", -1);
			int posy = tempAttr.GetInt("lastposY", -1);
			int posz = tempAttr.GetInt("lastposZ", -1);
			BlockPos pos = blockSel.Position;
			float treeResistance;
			if (pos.X != posx || pos.Y != posy || pos.Z != posz || counter % 30 == 0)
			{
				int baseResistance;
				int woodTier;
				this.FindTree(player.Entity.World, pos, out baseResistance, out woodTier);
				if (this.ToolTier < woodTier - 3)
				{
					return remainingResistance;
				}
				treeResistance = (float)Math.Max(1.0, Math.Sqrt((double)baseResistance / 1.45));
				tempAttr.SetFloat("treeResistance", treeResistance);
			}
			else
			{
				treeResistance = tempAttr.GetFloat("treeResistance", 1f);
			}
			tempAttr.SetInt("lastposX", pos.X);
			tempAttr.SetInt("lastposY", pos.Y);
			tempAttr.SetInt("lastposZ", pos.Z);
			return base.OnBlockBreaking(player, blockSel, itemslot, remainingResistance, dt / treeResistance, counter);
		}

		// Token: 0x060017A2 RID: 6050 RVA: 0x000D8BEC File Offset: 0x000D6DEC
		public override bool OnBlockBrokenWith(IWorldAccessor world, Entity byEntity, ItemSlot itemslot, BlockSelection blockSel, float dropQuantityMultiplier = 1f)
		{
			IPlayer byPlayer = null;
			if (byEntity is EntityPlayer)
			{
				byPlayer = byEntity.World.PlayerByUid(((EntityPlayer)byEntity).PlayerUID);
			}
			WeatherSystemBase modSystem = this.api.ModLoader.GetModSystem<WeatherSystemBase>(true);
			double windspeed = (modSystem != null) ? modSystem.WeatherDataSlowAccess.GetWindSpeed(byEntity.SidedPos.XYZ) : 0.0;
			int num;
			int woodTier;
			Stack<BlockPos> foundPositions = this.FindTree(world, blockSel.Position, out num, out woodTier);
			if (foundPositions.Count == 0)
			{
				return base.OnBlockBrokenWith(world, byEntity, itemslot, blockSel, dropQuantityMultiplier);
			}
			bool damageable = this.DamagedBy != null && this.DamagedBy.Contains(EnumItemDamageSource.BlockBreaking);
			float leavesMul = 1f;
			float leavesBranchyMul = 0.8f;
			int blocksbroken = 0;
			while (foundPositions.Count > 0)
			{
				BlockPos pos = foundPositions.Pop();
				blocksbroken++;
				Block block = world.BlockAccessor.GetBlock(pos);
				bool isLog = block.BlockMaterial == EnumBlockMaterial.Wood;
				bool isBranchy = block.Code.Path.Contains("branchy");
				bool isLeaves = block.BlockMaterial == EnumBlockMaterial.Leaves;
				world.BlockAccessor.BreakBlock(pos, byPlayer, isLeaves ? leavesMul : (isBranchy ? leavesBranchyMul : 1f));
				if (world.Side == EnumAppSide.Client)
				{
					axemechanical.dustParticles.Color = block.GetRandomColor(world.Api as ICoreClientAPI, pos, BlockFacing.UP, -1);
					axemechanical.dustParticles.Color |= -16777216;
					axemechanical.dustParticles.MinPos.Set((double)pos.X, (double)pos.Y, (double)pos.Z);
					if (block.BlockMaterial == EnumBlockMaterial.Leaves)
					{
						axemechanical.dustParticles.GravityEffect = (float)world.Rand.NextDouble() * 0.1f + 0.01f;
						axemechanical.dustParticles.ParticleModel = EnumParticleModel.Quad;
						axemechanical.dustParticles.MinVelocity.Set(-0.4f + 4f * (float)windspeed, -0.4f, -0.4f);
						axemechanical.dustParticles.AddVelocity.Set(0.8f + 4f * (float)windspeed, 1.2f, 0.8f);
					}
					else
					{
						axemechanical.dustParticles.GravityEffect = 0.8f;
						axemechanical.dustParticles.ParticleModel = EnumParticleModel.Cube;
						axemechanical.dustParticles.MinVelocity.Set(-0.4f + (float)windspeed, -0.4f, -0.4f);
						axemechanical.dustParticles.AddVelocity.Set(0.8f + (float)windspeed, 1.2f, 0.8f);
					}
					world.SpawnParticles(axemechanical.dustParticles, null);
				}
				if (damageable && isLog)
				{
					this.DamageItem(world, byEntity, itemslot, 1);
				}
				if (itemslot.Itemstack == null)
				{
					return true;
				}
				if (isLeaves && leavesMul > 0.03f)
				{
					leavesMul *= 0.85f;
				}
				if (isBranchy && leavesBranchyMul > 0.015f)
				{
					leavesBranchyMul *= 0.7f;
				}
			}
			if (blocksbroken > 35)
			{
				Vec3d pos2 = blockSel.Position.ToVec3d().Add(0.5, 0.5, 0.5);
				this.api.World.PlaySoundAt(new AssetLocation("sounds/effect/treefell"), pos2.X, pos2.Y, pos2.Z, byPlayer, false, 32f, GameMath.Clamp((float)blocksbroken / 100f, 0.25f, 1f));
			}
			return true;
		}

		// Token: 0x060017A3 RID: 6051 RVA: 0x000D8F54 File Offset: 0x000D7154
		public Stack<BlockPos> FindTree(IWorldAccessor world, BlockPos startPos, out int resistance, out int woodTier)
		{
			Queue<Vec4i> queue = new Queue<Vec4i>();
			HashSet<BlockPos> checkedPositions = new HashSet<BlockPos>();
			Stack<BlockPos> foundPositions = new Stack<BlockPos>();
			resistance = 0;
			woodTier = 0;
			Block block = world.BlockAccessor.GetBlock(startPos);
			if (block.Code == null)
			{
				return foundPositions;
			}
			JsonObject attributes = block.Attributes;
			string treeFellingGroupCode = (attributes != null) ? attributes["treeFellingGroupCode"].AsString(null) : null;
			JsonObject attributes2 = block.Attributes;
			int spreadIndex = (attributes2 != null) ? attributes2["treeFellingGroupSpreadIndex"].AsInt(0) : 0;
			JsonObject attributes3 = block.Attributes;
			if (attributes3 != null && !attributes3["treeFellingCanChop"].AsBool(true))
			{
				return foundPositions;
			}
			EnumTreeFellingBehavior bh = EnumTreeFellingBehavior.Chop;
			ICustomTreeFellingBehavior ctfbh = block as ICustomTreeFellingBehavior;
			if (ctfbh != null)
			{
				bh = ctfbh.GetTreeFellingBehavior(startPos, null, spreadIndex);
				if (bh == EnumTreeFellingBehavior.NoChop)
				{
					resistance = foundPositions.Count;
					return foundPositions;
				}
			}
			if (spreadIndex < 2)
			{
				return foundPositions;
			}
			if (treeFellingGroupCode == null)
			{
				return foundPositions;
			}
			queue.Enqueue(new Vec4i(startPos.X, startPos.Y, startPos.Z, spreadIndex));
			foundPositions.Push(startPos);
			checkedPositions.Add(startPos);
			while (queue.Count > 0 && foundPositions.Count <= 2500)
			{
				Vec4i pos = queue.Dequeue();
				block = world.BlockAccessor.GetBlock(pos.X, pos.Y, pos.Z);
				ICustomTreeFellingBehavior ctfbhh = block as ICustomTreeFellingBehavior;
				if (ctfbhh != null)
				{
					bh = ctfbhh.GetTreeFellingBehavior(startPos, null, spreadIndex);
				}
				if (bh != EnumTreeFellingBehavior.NoChop)
				{
					for (int i = 0; i < Vec3i.DirectAndIndirectNeighbours.Length; i++)
					{
						Vec3i facing = Vec3i.DirectAndIndirectNeighbours[i];
						BlockPos neibPos = new BlockPos(pos.X + facing.X, pos.Y + facing.Y, pos.Z + facing.Z);
						float num = GameMath.Sqrt(neibPos.HorDistanceSqTo((double)startPos.X, (double)startPos.Z));
						float vertdist = (float)(neibPos.Y - startPos.Y);
						float f = (bh == EnumTreeFellingBehavior.ChopSpreadVertical) ? 0.5f : 2f;
						if (num - 1f < f * vertdist && !checkedPositions.Contains(neibPos))
						{
							block = world.BlockAccessor.GetBlock(neibPos);
							if (!(block.Code == null) && block.Id != 0)
							{
								JsonObject attributes4 = block.Attributes;
								if (!(((attributes4 != null) ? attributes4["treeFellingGroupCode"].AsString(null) : null) != treeFellingGroupCode))
								{
									JsonObject attributes5 = block.Attributes;
									int nspreadIndex = (attributes5 != null) ? attributes5["treeFellingGroupSpreadIndex"].AsInt(0) : 0;
									if (pos.W >= nspreadIndex)
									{
										checkedPositions.Add(neibPos);
										if (bh != EnumTreeFellingBehavior.ChopSpreadVertical || facing.Equals(0, 1, 0) || nspreadIndex <= 0)
										{
											resistance += nspreadIndex + 1;
											if (woodTier == 0)
											{
												woodTier = nspreadIndex;
											}
											foundPositions.Push(neibPos.Copy());
											queue.Enqueue(new Vec4i(neibPos.X, neibPos.Y, neibPos.Z, nspreadIndex));
										}
									}
								}
							}
						}
					}
				}
			}
			return foundPositions;
		}

		// Token: 0x04000CDB RID: 3291
		private static SimpleParticleProperties dustParticles = new SimpleParticleProperties
		{
			MinPos = new Vec3d(),
			AddPos = new Vec3d(),
			MinQuantity = 0f,
			AddQuantity = 3f,
			Color = ColorUtil.ToRgba(100, 200, 200, 200),
			GravityEffect = 1f,
			WithTerrainCollision = true,
			ParticleModel = EnumParticleModel.Quad,
			LifeLength = 0.5f,
			MinVelocity = new Vec3f(-1f, 2f, -1f),
			AddVelocity = new Vec3f(2f, 0f, 2f),
			MinSize = 0.07f,
			MaxSize = 0.1f,
			WindAffected = true
		};
	}
}

